# copyright 2012-2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

from cubicweb.predicates import is_instance, match_form_params
from cubicweb.web.views.primary import PrimaryView

from cubes.wireit.views.mixins import WiringViewMixin


class WiringPrimaryView(WiringViewMixin, PrimaryView):
    __select__ = (PrimaryView.__select__ & is_instance('Wiring')
                  & ~ match_form_params('__nowireit'))

    def render_entity(self, entity):
        self.render_entity_title(entity)
        self.setup_editor(entity, entity.language[0])

    def wiring_descr_html(self, wiring, wlang, editable):
        w, html = self.w, []
        self.w = html.append
        self.render_entity_attributes(wiring)
        self.w = w
        return u''.join(html)




# -*- coding: utf-8 -*-
# copyright 2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os.path as osp
from json import loads, dumps

from cubicweb import ValidationError
from cubicweb.devtools.testlib import CubicWebTC

HERE = osp.dirname(__file__)


class WiringTCMixin(object):

    def setup_database(self):
        super(WiringTCMixin, self).setup_database()
        with self.admin_access.cnx() as cnx:
            self.wl_eid = cnx.create_entity('WiringLanguage', name=u'wl').eid
            cnx.commit()


class WiringAddHookTC(WiringTCMixin, CubicWebTC):

    def test_name(self):
        'creating a wiring with no name should not crash but set it'
        with self.admin_access.cnx() as cnx:
            wiring = cnx.create_entity('Wiring', language=self.wl_eid,
                                       json=u'')
            cnx.commit()
            self.assertTrue(wiring.name)


class UpdateWiringLanguageHookTC(WiringTCMixin, CubicWebTC):

    def setup_database(self):
        super(UpdateWiringLanguageHookTC, self).setup_database()
        with self.admin_access.cnx() as cnx:
            wl = cnx.entity_from_eid(self.wl_eid)
            wl.cw_set(json=self._json_content('wlan.json'))
            self.wiring_eid = cnx.create_entity(
                'Wiring', json=self._json_content('wiring.json'),
                language=self.wl_eid).eid
            cnx.commit()

    def _json_content(self, fname):
        json = loads(open(osp.join(HERE, 'data', fname)).read())
        return unicode(dumps(json))

    def test_valid(self):
        'check a valid definition does not raise ValidationError'
        with self.admin_access.cnx() as cnx:
            wiring = cnx.entity_from_eid(self.wiring_eid)
            new_def = loads(wiring.json)
            new_def['toto'] = u'titi'
            wiring.cw_set(json=unicode(dumps(new_def)))
            cnx.commit()

    def test_invalid_module_list(self):
        'remove a module used by the wiring def and check ValidationError'
        with self.admin_access.cnx() as cnx:
            wl = cnx.entity_from_eid(self.wl_eid)
            wldef = loads(wl.json)
            del wldef['modules'][0]
            with self.assertRaises(ValidationError) as cm:
                wl.cw_set(json=unicode(dumps(wldef)))
                cnx.commit()
            cnx.rollback()
        self.assertIn('Invalid WiringLanguage change',
                      str(cm.exception))

    def test_invalid_module_definition(self):
        'change a container def used by the wiring and check ValidationError'
        with self.admin_access.cnx() as cnx:
            wl = cnx.entity_from_eid(self.wl_eid)
            wldef = loads(wl.json)
            wldef['modules'][0]['container']['title'] = u'changed_title'
            with self.assertRaises(ValidationError) as cm:
                wl.cw_set(json=unicode(dumps(wldef)))
                cnx.commit()
            cnx.rollback()
        self.assertIn('Invalid WiringLanguage change', str(cm.exception))


if __name__ == '__main__':
    import unittest
    unittest.main()



# copyright 2012-2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

'''mixins shared by several views'''

from json import dumps, loads


class WiringViewMixin(object):
    '''mixin to be used to display a wiring inside a wireit editor interface'''

    css = ('wireit/lib/yui/assets/skins/sam/skin.css',
           'wireit/lib/inputex/css/inputEx.css',
           'wireit/css/WireIt.css',
           'wireit/css/WireItEditor.css',
           'cw_wiring_editor.css',
           )
    js = ('wireit/lib/yui/utilities/utilities.js',
          'wireit/lib/yui/resize/resize-min.js',
          'wireit/lib/yui/layout/layout-min.js',
          'wireit/lib/yui/container/container-min.js',
          'wireit/lib/yui/json/json-min.js',
          'wireit/lib/yui/button/button-min.js',
          'wireit/lib/yui/tabview/tabview-min.js',
          'wireit/lib/inputex/js/inputex.js',
          'wireit_patches/lib/inputex/js/Field.js',
          'wireit_patches/js/util/inputex/WirableField-beta.js',
          'wireit/lib/inputex/js/Group.js',
          'wireit/lib/inputex/js/Visus.js',
          'wireit/lib/inputex/js/fields/StringField.js',
          'wireit/lib/inputex/js/fields/NumberField.js',
          'wireit/lib/inputex/js/fields/Textarea.js',
          'wireit/lib/inputex/js/fields/SelectField.js',
          'wireit/lib/inputex/js/fields/EmailField.js',
          'wireit/lib/inputex/js/fields/UrlField.js',
          'wireit/lib/inputex/js/fields/ListField.js',
          'wireit/lib/inputex/js/fields/CheckBox.js',
          'wireit/lib/inputex/js/fields/InPlaceEdit.js',
          'wireit_patches/lib/inputex/js/fields/FileField-beta.js',
          'wireit/js/WireIt.js',
          'wireit/js/CanvasElement.js',
          'wireit/js/Wire.js',
          'wireit_patches/js/Terminal.js',
          'wireit/js/util/DD.js',
          'wireit/js/util/DDResize.js',
          'wireit_patches/js/Container.js',
          'wireit/js/Layer.js',
          'wireit/js/util/inputex/FormContainer-beta.js',
          'wireit/js/LayerMap.js',
          'wireit_patches/js/WiringEditor.js',
          'wireit/js/ImageContainer.js',
          'wireit/js/InOutContainer.js',
          )

    template = """
<div id="wireiteditor" class="%(css)s">
    <div id="left">
        <div id="modules"></div>
        <div class="section">
            <div class="iformTitle">
                <span>%(minimap)s</span>
            </div>
            <div style="position: relative; margin-top: 0.5em">
                <div id="layerMap"></div>
            </div>
        </div>
        <div class="section">
            %(wiring_descr)s
        </div>
    </div>
    <div id="center"></div>
</div>
"""

    def add_media(self):
        self._cw.add_js('cw_wiring_editor.js')
        self._cw.add_js(('jq_upload/js/jquery.iframe-transport.js',
                         'jq_upload/js/vendor/jquery.ui.widget.js',
                         'jq_upload/js/jquery.fileupload.js',))
        for css in self.css:
            self._cw.add_css(self._cw.data_url(css), localfile=False)
        for js in self.js:
            self._cw.add_js(self._cw.data_url(js), localfile=False)
        if self._cw.ie_browser():
            self._cw.add_js('excanvas.js')

    def html(self, wiring, wlang, editable, cancel_url):
        'return view html structure'
        ctx = {'css': 'editable' if editable else 'readonly',
               'minimap': self._cw._('minimap'),
               'wiring_descr': self.wiring_descr_html(wiring, wlang, editable),
               }
        return self.template % ctx

    def wiring_descr_html(self, wiring, wlang, editable):
        'this is the place where you can add a html description of the entity'
        return u''

    def set_wlang_permissions(self, wlang, editable):
        '''return wiring language `wlang` json description modified to
        graphically show edition is not possible'''
        json = wlang.cw_adapt_to('wireit.json').json
        if editable:
            return json
        descr = loads(json)
        for module in descr['modules']:
            for field in module['container']['fields']:
                field['inputParams']['disabled'] = True # disallow field edition
                field['inputParams']['editable'] = False # disallow terminal edition
        return dumps(descr)

    def set_wiring_edit_permissions(self, wiring, globally_editable):
        """ hook to allow fine-tuning the editability of the langage itself """
        return wiring.cw_adapt_to('wireit.json').json

    def setup_editor(self, wiring, wlang, editable=False, cancel_url='/'):
        'setup the entire wireit editor'
        self.add_media()
        self.w(self.html(wiring, wlang, editable, cancel_url))
        wlang_json = self.set_wlang_permissions(wlang, editable)
        try:
            eid = int(wiring.eid)
        except ValueError: # entity creation
            eid = None
        json_wiring = self.set_wiring_edit_permissions(wiring,
                                                       globally_editable=editable)
        ctx = {'wiring_eid': dumps(eid),
               'wiring_json': dumps(json_wiring or '{}'),
               'lang_eid': dumps(wlang.eid),
               'lang_json': dumps(wlang_json),}
        self._cw.add_onload('cw.wireiteditor_setup(%(wiring_eid)s, %(wiring_json)s, '
                            '%(lang_eid)s, %(lang_json)s);' % ctx)



# -*- coding: utf-8 -*-
# copyright 2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-wireit forms"""

from cubicweb.predicates import is_instance, match_form_params
from cubicweb.schema import META_RTYPES

from cubicweb.predicates import is_instance, match_form_params
from cubicweb.web import formwidgets as fwdgs
from cubicweb.web.views import uicfg


# custom uicfg

class WireItWiringCustomUicfgMixin(object):
    __select__ = is_instance('Wiring') & ~match_form_params('__nowireit')

class WireItWiringAfs(WireItWiringCustomUicfgMixin, uicfg.AutoformSectionRelationTags):
    pass

class WireItWiringAffk(WireItWiringCustomUicfgMixin, uicfg.AutoformFieldKwargsTags):
    pass

class WireItWiringPvs(WireItWiringCustomUicfgMixin, uicfg.PrimaryViewSectionRelationTags):
    pass

try:
    wireit_afs = WireItWiringAfs(__module__=__name__)
    wireit_affk = WireItWiringAffk(__module__=__name__)
    wireit_pvs = WireItWiringPvs(__module__=__name__)
except TypeError:
    # logilab-common < 1.4.0, where RegistrableInstance does not make use of
    # __module__
    wireit_afs = WireItWiringAfs()
    wireit_affk = WireItWiringAffk()
    wireit_pvs = WireItWiringPvs()

# default uicfg

_abaam = uicfg.actionbox_appearsin_addmenu
_abaam.tag_object_of(('Wiring', 'language', 'WiringLanguage'), True)

# custom uicfg


wireit_affk.tag_attribute(('Wiring', 'name'),
                          {'widget': fwdgs.TextInput(attrs=dict(size=25))})
wireit_affk.tag_attribute(('Wiring', 'json'),
                          {'widget': fwdgs.HiddenInput})
wireit_affk.tag_subject_of(('Wiring', 'language', 'WiringLanguage'),
                           {'widget': fwdgs.HiddenInput})


wireit_pvs.tag_attribute(('Wiring', 'json'), 'hidden')
for rtype in META_RTYPES:
    wireit_pvs.tag_subject_of(('*', rtype, '*'), 'hidden')
    wireit_pvs.tag_object_of(('*', rtype, '*'), 'hidden')



# copyright 2012-2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

from logilab.common.registry import objectify_predicate

from cubicweb.predicates import (is_instance, match_form_params,
                                 specified_etype_implements)
from cubicweb.web import stdmsgs
from cubicweb.web.formwidgets import Button, SubmitButton
from cubicweb.web.views.editforms import EditionFormView, CreationFormView

from cubes.wireit.views.mixins import WiringViewMixin


class WiringFormViewMixin(WiringViewMixin):

    def form_title(self, ent):
        """use h1 instead of more sophisticated css-driven design
        that would required cw.form.css"""
        ptitle = self._cw._(self.title)
        self.w(u'<h1>%s %s</h1>' % (ent.dc_type(), ptitle and '(%s)' % ptitle))

    def wiring_descr_html(self, wiring, wlang, editable):
        form = self._cw.vreg['forms'].select(self.form_id, self._cw,
                                             entity=wiring,
                                             submitmsg=self.submited_message())
        form.form_buttons = [SubmitButton(),
                             Button(stdmsgs.BUTTON_CANCEL, cwaction='cancel')]
        return form.render(w=None)


class WiringEditionFormView(WiringFormViewMixin, EditionFormView):
    __select__ = (EditionFormView.__select__ & is_instance('Wiring')
                  & ~ match_form_params('__nowireit'))

    def render_form(self, entity):
        self.form_title(entity)
        self.setup_editor(entity, entity.language[0],
                          editable=True, cancel_url=entity.absolute_url())


def _wlang_linkto_eid(req):
    wlang_eid = None
    for linkto in req.list_form_param('__linkto'):
        rtype, linkto_eid, role = linkto.split(':')
        if rtype == 'language':
            wlang_eid = linkto_eid
            break
    return wlang_eid


@objectify_predicate
def has_link_to_wlang(self, req, **kwargs):
    return _wlang_linkto_eid(req) is not None


class WiringCreationFormView(WiringFormViewMixin, CreationFormView):
    __select__ = (CreationFormView.__select__ & specified_etype_implements('Wiring')
                  & ~match_form_params('__nowireit') & has_link_to_wlang())

    def render_form(self, wiring):
        self.form_title(wiring)
        wlang_eid = _wlang_linkto_eid(self._cw)
        wlang = self._cw.entity_from_eid(int(wlang_eid))
        assert wlang.e_schema.type == 'WiringLanguage'
        self.setup_editor(wiring, wlang, editable=True)



# -*- coding: utf-8 -*-
# copyright 2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-wireit specific hooks and operations"""

from json import loads

from cubicweb import ValidationError
from cubicweb.predicates import is_instance
from cubicweb.server.hook import Hook, Operation, DataOperationMixIn


class BeforeAddWiringHook(Hook):
    __regid__ = 'wireit.before_add_wiring'
    __select__ = Hook.__select__ & is_instance('Wiring')
    events = ('before_add_entity',)

    def __call__(self):
        self.entity.cw_edited.setdefault('name', (
            self._cw._(u'wiring %s') % self.entity.eid))


class WiringValidityCheckOp(DataOperationMixIn, Operation):
    'Checks the validity of all wirings attached to a (e.g. edited) language'

    def module_def(self, module_name, wlang_json):
        '''
        Return module of name `module_name` definition in the definition of
        the wiring language definition `wlang_def`, or None if not found.
        '''
        for module in loads(wlang_json)['modules']:
            if module['name'] == module_name:
                return module

    def precommit_event(self):
        '''
        Minimal semi-incremental check for the validity of all wirings of
        a wiring language:

        * check the modules of a wiring are present in the language definition
        * suppose the wirings were valid for the old version of the language and
          check the new version does not change the module definitions used by
          the wirings.
        '''
        for wlang, old_json in self.get_data():
            for wiring in wlang.reverse_language:
                used_modules = wiring.module_names
                if not used_modules.issubset(wlang.module_names):
                    msg = wiring._cw._('Invalid WiringLanguage change')
                    raise ValidationError(wlang.eid, {'json': msg})
                for module in used_modules:
                    if (self.module_def(module, wlang.json)
                        != self.module_def(module, old_json)):
                        msg = wiring._cw._('Invalid WiringLanguage change')
                        raise ValidationError(wlang.eid, {'json': msg})


class BeforeUpdateWiringLanguageHook(Hook):
    __regid__ = 'wireit.before_update_language'
    __select__ = Hook.__select__ & is_instance('WiringLanguage')
    events = ('before_update_entity',)

    def __call__(self):
        wl = self.entity
        if 'json' in wl.cw_edited:
            old, new = wl.cw_edited.oldnewvalue('json')
            if not old is None:
                WiringValidityCheckOp.get_instance(self._cw).add_data((wl, old))



# copyright 2012-2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-wireit controllers"""

from cubicweb import Binary
from cubicweb.predicates import match_form_params, match_user_groups
from cubicweb.web import jsonize
from cubicweb.web.controller import Controller


class FileFieldUploadController(Controller):
    __regid__ = 'wireit_upload'
    __select__ = (Controller.__select__
                  & match_user_groups('managers', 'users')
                  & match_form_params('files[]'))

    @jsonize
    def publish(self, *args, **kwargs):
        self.debug("UploadController call with args %s, kwargs %s and form %s",
                   args, kwargs, self._cw.form)
        try:
            data_name, flike_obj = self._cw.form['files[]']
            newfile = self._cw.create_entity('File', data_name=data_name,
                                             data=Binary(flike_obj.read()))
        except Exception as exc:
            self.exception(exc)
            return {'status': False, 'msg': self._cw._('unknown error')}
        return {
            'status': True,
            'eid': newfile.eid,
            'title': newfile.dc_title(),
            'url': newfile.absolute_url(),
        }



# copyright 2012-2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-wireit actions"""

from cubicweb.predicates import match_user_groups, is_instance
from cubicweb.web.action import Action

from six import text_type

try:
    from cubicweb import _
except ImportError:
    # pre 3.22
    _ = text_type


class WiringAdminViewSourceJson(Action):
    """action for admins who want to see the json source code of a wiring"""

    __regid__ = 'processing.wiring_admin_view'
    __select__ = (Action.__select__ & match_user_groups('managers')
                  & is_instance('Wiring'))

    title = _('view source code')
    category = 'mainactions'

    def url(self):
        wiring = self.cw_rset.get_entity(0, 0)
        return wiring.absolute_url(__nowireit='')



# -*- coding: utf-8 -*-
# copyright 2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-wireit entity's classes"""
from cubicweb.predicates import is_instance
from cubicweb.entities import AnyEntity
from cubicweb.view import EntityAdapter

from json import loads

class Wiring(AnyEntity):
    __regid__ = 'Wiring'

    @property
    def wlang(self):
        return self.language[0]

    @property
    def module_names(self):
        if self.json:
            return frozenset(m['name'] for m in loads(self.json).get('modules'))
        return ()

class WiringLanguage(AnyEntity):
    __regid__ = 'WiringLanguage'

    @property
    def module_names(self):
        if self.json:
            return frozenset(m['name'] for m in loads(self.json).get('modules'))
        return ()

class WireitJson(EntityAdapter):
    __regid__ = 'wireit.json'
    __select__ = is_instance('Wiring', 'WiringLanguage')

    @property
    def json(self):
        return self.entity.json



# -*- coding: utf-8 -*-
# copyright 2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""simple views (typically one line or so) for cubicweb-wireit"""

from cubicweb.predicates import is_instance
from cubicweb.web.views.baseviews import InContextView


class UploadedFileInContextView(InContextView):
    __regid__ = 'wireit_fileincontext'
    __select__ = InContextView.__select__ & is_instance('File')

    def cell_call(self, row, col):
        entity = self.cw_rset.get_entity(row, col)
        super(UploadedFileInContextView, self).cell_call(row, col)
        self.w('<img src="%s" alt="%s"/>' % (
            entity.icon_url(), self._cw._('icon for %s') % entity.data_format))



# -*- coding: utf-8 -*-
# copyright 2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-wireit schema"""

from yams.buildobjs import EntityType, SubjectRelation, String

from six import text_type

try:
    from cubicweb import _
except ImportError:
    # pre 3.22
    _ = text_type


class WiringLanguage(EntityType):
    name = String(required=True, maxsize=128, description=_('name of the wiring language'))
    json = String(description=_('json description of the wiring language as used by WireIt itself'))


class Wiring(EntityType):
    name = String(required=True, maxsize=128, description=_('name of the Wiring'))
    language = SubjectRelation('WiringLanguage', cardinality='1*', inlined=True, composite='object',
                               description=_('Wiring language used to define this wiring'))
    json = String(description=_('json description of the wiring as serialized by WireIt itself'))



# pylint: disable=W0622
"""cubicweb-wireit application packaging information"""

modname = 'wireit'
distname = 'cubicweb-wireit'

numversion = (0, 6, 0)
version = '.'.join(str(num) for num in numversion)

license = 'LGPL'
author = 'LOGILAB S.A. (Paris, FRANCE)'
author_email = 'contact@logilab.fr'
description = 'WireIt workflow store for the CubicWeb framework'
web = 'http://www.cubicweb.org/project/%s' % distname

__depends__ = {'cubicweb': '>= 3.16.0',
               'six': '>= 1.4.0',
               'cubicweb-file': None}
__recommends__ = {}

classifiers = [
    'Environment :: Web Environment',
    'Framework :: CubicWeb',
    'Programming Language :: Python',
    'Programming Language :: JavaScript',
    ]

